import socket
import sys
import threading
import time

class Transport:
    def __init__(self):
        self.lock = threading.Lock()
        self.sock = None
        self.serverAddr = None
        self.clientAddr = None
        self.clientSock = None

    def init(self, port, addr):
        self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.serverAddr = (addr, int(port))

        try:
            self.sock.bind(self.serverAddr)
        except socket.error:
            print("[uarm_serial] - Failed to bind socket.")
            raise

        self.sock.listen(1)

        try:
            self.clientSock, self.clientAddr = self.sock.accept()
        except socket.error:
            print("[uarm_serial] - Failed to accept socket connection.")
            raise

    def receive(self):
        return self.clientSock.recv(4096).decode()

    def send(self, message):
        self.lock.acquire()

        try:
            message = message + "\n"
            self.clientSock.sendall(message.encode())
        except IOError:
            print("[uarm_serial] - Socket transmission failed.")
        finally:
            self.lock.release()

    def close(self):
        self.lock.acquire()

        try:
            self.sock.close()
            self.clientSock.close()
        except socket.error:
            print("[uarm_serial] - Failed to close socket.")
            raise
        finally:
            self.lock.release()



# Imports
import time
import queue
from threading import Thread
# Local imports
from .transporter import Transport

class Communicator(Thread):

    def __init__(self, uarm, port, addr):
        Thread.__init__(self)
        self.interrupted = False
        self.daemon = True
        self.setName("uARM_Comm")
        self.uarm = uarm
        self.transport = Transport()
        self.transport.init(port, addr)
        self.command_queue = queue.Queue()

    def subscribeLoop(self):
        while not self.is_interrupted():
            try:
                self.parse_update(self.transport.receive())
            except IOError:
                print("[uarm_serial] - Parser failed.")
                self.interrupted = True
                raise

    def publishLoop(self):
        while not self.is_interrupted():
            try:
                self.sendAll()
                time.sleep(1) # TODO: is needed?
            except IOError:
                print("[uarm_serial] - Parser failed.")
                self.interrupted = True
                raise

    def parse_update(self, message):
        if message == "":
            return

        op = int(message)

        if op == 0: # Idle
            self.uarm.set_idle()
        elif op == 1: # Running
            self.uarm.set_running()
        else:
            print("[uarm_serial] - Unknown command.")

    def is_interrupted(self):
        return self.interrupted

    def set_interrupted(self, interrupted):
        self.interrupted = interrupted

    """
    Add a message to the queue for transmission
    """
    def send(self, message):
        self.command_queue.put(message)

    def sendAll(self):
        while not self.command_queue.empty():
            self.transport.send(self.command_queue.get())



# Imports
import threading
import serial
import os
import sys
import time
import random
# import logging
from uf.wrapper.swift_api import SwiftAPI
from .transporter import Transport
from .comm import Communicator

class UARM:
    # Camera (OpenMV)
    camera = None
    # uARM
    uarm = None
    # Position
    xPos = 0
    yPos = 0
    zPos = 170
    # Timing
    startTime = 0
    # Control
    mode = 0 # Idle = 0, Running = 1
    # Locks
    moveLock = threading.Lock()

    def __init__(self, camera_port="COM4", uarm_port="COM3", serial_baudrate=115200, serial_timeout=1):
        # Setup camera serial
        self.camera = serial.Serial(camera_port, baudrate=serial_baudrate, timeout=serial_timeout)
        # Setup uARM
        self.uarm = SwiftAPI()
        self.uarm.set_position(self.xPos, self.yPos, self.zPos, speed=250)
        self.flush_cmd()

    def get_uarm(self):
        return self.uarm

    def get_camera(self):
        return self.camera

    def in_idle_pos(self):
        return (self.xPos == 0 and self.yPos == 0 and self.zPos == 170)

    def set_idle(self):
        self.camera.write("Ix".encode("ascii"))

    def go_to_idle(self):
        self.moveLock.acquire()

        try:
            self.xPos = 0
            self.yPos = 0
            self.zPos = 170
            self.uarm.set_position(x=self.xPos, y=self.yPos, z=self.zPos, speed=500000, wait=True)
        finally:
            self.moveLock.release()

        time.sleep(3)

    def run(self, func):
        self.mode = 1
        #self.camera.write("Rx".encode("ascii")) # Send 'Rx' to camera for running

        while self.mode == 1 and self.uarm.connected:
            lambda arg: func(arg)
        else:
            if not self.in_idle_pos():
                self.set_idle()
                self.go_to_idle()
